%----- Compare models by Akaike's AICc : individual fittings versus global one ----
% by Evgenii Kovrigin    3/7/08
% For details of model testing see  {Motulsky, 2004 #4251}
% 1.	Motulsky, H. and Christopoulos, A., Fitting Models to Biological Data Using 
% Linear and Nonlinear Regression: A Practical Guide to Curve Fitting. 1 edition ed. 2004: 
% Oxford University Press, USA. 352 
%
% To use: 
%      - Run  individual and global fitting for a group of residues 
%         NOTE: make sure fits are reasonable!
%
%      - Fit_grid_ek.m saves the sum of resudual squares and number of points in
%         the files resname_SS.txt. Look up composite name for the global
%         fitting session in grid_results... file.
%
%      -  put necessary file names into AIC_setup on local folder with
%      ...SS.txt files.
%
%      -  execute 'compare.m'. Results are displayed and saved into plain
%      text file.

AIC_setup



%--- reading SS from individual fits results ---
res_number=size(resnames);
res_number=res_number(1); %-- because it is 'char'---
SSindiv=zeros(1,res_number); %-- collect here sum of squares from individual fits
nTindiv=zeros(1,res_number); %-- collect here number of titration points from individual fits
fprintf('Read individual fits:\n\t\tN\tSS\n');
for i=1:res_number
    resname=deblank(resnames(i,:));
    fname=sprintf('%s.SS.txt',resname);
    fp=fopen(fullfile('results','SS',fname),'r');
    data=fscanf(fp,'%d%e');
    N=data(1);
    SS=data(2);
    disp(sprintf('%s\t%d\t%e\t',resname,N,SS));
    SSindiv(i)=SS;
    nTindiv(i)=N;
end
SStotal=sum(SSindiv);
nTtotal=sum(nTindiv);  %-- number of data points

%--- reading global fit result ---
resname=deblank(global_resname);
fname=sprintf('%s.SS.txt',resname);
fp=fopen(fullfile('results','SS',fname),'r');
data=fscanf(fp,'%d%e');
SSglobal=data(2);
disp(sprintf('%s\t%d\t%e\t\n\n',resname,nTtotal,SSglobal));




%% Akaike criterion testing (see {Motulsky, 2004 #4251})
disp('Performing evaluation of likelyhood for each model by Akaike Information Criterion');

SSindividual=SStotal;
Kindividual=Num_Fit_Pars*res_number + 1;
Kglobal=Num_Fit_Pars + 1;
N=nTtotal;

%-- check applicability
if Kindividual > (N-2)
    error('Too few experimental points for Akaike test! N must be at least 2 larger than K!:\nK=%f, N=%f', Kindividual, N)
end

%--model 1:  AICc for global Koffs
AICc_1= AICc(N, Kglobal, SSglobal);

%-- model 2: AICc for individual Koffs
AICc_2= AICc(N, Kindividual, SSindividual);


if AICc_1 > AICc_2
    conclusion='Model 2 is more correct';
else
    conclusion='Model 1 is more correct';
end


%-- Evidence ratio : probability of correct 1/ probability of correct 2
EvRat=1/exp(-0.5*(AICc_2-AICc_1));

%--- output for statistical model testing ----
fp=fopen(outfname,'W');
outstring1=sprintf('1st Model (global fitting)\nN = %d\nK=%d\nSS = %g\nAICc=%g\n',N, Kglobal, SSglobal, AICc_1);
outstring2=sprintf('2nd Model (individual fitting)\nN = %d\nK=%d\nSS = %g\nAICc=%g\n',N, Kindividual, SSindividual, AICc_2);
outstring3=sprintf('\nEvidence ratio (model 1 is more likely than model 2 by factor of): %.1e\n',EvRat);

disp(outstring1);disp(outstring2);disp(conclusion);disp(outstring3);
fprintf(fp,'%s\n%s\n%s\n%s ',outstring1,outstring2,conclusion,outstring3);

fclose(fp);
disp(sprintf('Results are written to %s', outfname));