
%% Load parameters of simulation
titration_plan_setup
% load model parameters (Kd and Koff)
model_setup



%% Convert to SI units
P0=P0*1e-3; % M
VP0=VP0*1e-3; % L
dCL=dCL.*1e-3; % M
dVL=dVL.*1e-3;%  L

% check lengths of arrays
N1=length(dVL);
N2=length(dCL);
if N1~=N2
    error('ERROR! Array of ligand stock volumes has %d elements while array of ligand stock concentrations has %d elements!',...
        N1, N2);
end


%% Compute total concentration of a receptor and a ligand/receptor ratio
N_additions=length(dVL);
P_current=zeros(1,N_additions+1); % Total concentration of a receptor during titration
V_current=zeros(1,N_additions+1); % Volume of a sample during titration
L_current=zeros(1,N_additions+1); % Concentration of ligand during titration
LP_ratio_current=zeros(1,N_additions+1); % a ligand/receptor ratio during titration
PLeq_current=zeros(1,N_additions+1);  % Equilibrium concentration of a complex
Peq_current=zeros(1,N_additions+1);  % Equilibrium concentration of a free protein
Leq_current=zeros(1,N_additions+1);  % Equilibrium concentration of a free ligand

% Set first point
P_current(1)=P0;
Peq_current(1)=P0;
V_current(1)=VP0;

% total volume of added ligand
dVL_total=0;

% Compute subsequent points
for counter=2:1:N_additions+1
    dVL_total=dVL_total+dVL(counter-1);
    V_total=dVL_total+VP0;
    

    V_current(counter)=V_total;
    L_current(counter)=(dVL(1:counter-1)*dCL(1:counter-1)')/V_total;
    P_current(counter)=P0*VP0/V_total;
    LP_ratio_current(counter)=L_current(counter)/P_current(counter);
    
    % compute equilibrium concentrations using P+L=PL model
    [Peq, Leq, PLeq]= simple_equilibrium(P_current(counter),L_current(counter),Model.Kd);
    
    PLeq_current(counter)=PLeq;
    Peq_current(counter)=Peq;
    Leq_current(counter)=Leq;
    
    

end

%% Prepare output folder
if ~exist(path_to_save, 'dir')
    mkdir(path_to_save)
    mkdir(fullfile(path_to_save, path_simulation_data));
end
if ~exist(fullfile(path_to_save, path_simulation_data), 'dir')
    mkdir(fullfile(path_to_save, path_simulation_data));
end

%% Plot calculated values
titration_points=1:1:N_additions+1;

[figure1, figure2]=create_titration_figure(project_name, titration_points, P_current, V_current, LP_ratio_current, ...
    PLeq_current, Peq_current, Leq_current);


saveas(figure1,fullfile(path_to_save, 'titration_plan.png'))
saveas(figure1,fullfile(path_to_save, 'titration_plan.fig'))
saveas(figure2, fullfile(path_to_save,'equilibrium.png'))
saveas(figure2, fullfile(path_to_save,'equilibrium.fig'))



%% Text output of results
fid=fopen(fullfile(path_to_save,'titration_plan.txt'), 'w');

fprintf(fid,'Project: %s\n\n\n', project_name);

fprintf(fid,'Total number of titration points (including ligand-free): %d\n\n', ...
    N_additions+1);


% protein concentration
fprintf(fid,'Protein concentration, M\n');
for counter=1:1:N_additions+1
    fprintf(fid, '%.4e\t', P_current(counter));
end
fprintf(fid, '\n\n');

fprintf(fid,'Ligand/Receptor ratio\n');
for counter=1:1:N_additions+1
    fprintf(fid, '%.4e\t', LP_ratio_current(counter));
end
fprintf(fid, '\n\n');


fclose(fid);


%% Prepare output for line-shape analysis
if ~(wPL>wP)
    error('ERROR!  wPL must be larger than wP for line-shape simulation !!!');
end

% Generate spectal arrays
spectral_points=100; % default number of points in simulated NMR line shapes
spectral_margin_FWHH=2 ; % default distance from peak maximum to the spectral edge

min_freq=wP  - spectral_margin_FWHH*R2_P;
max_freq=wPL  + spectral_margin_FWHH*R2_PL;
step=(max_freq-min_freq)/spectral_points;

freq=min_freq: step : max_freq;
intensity=1:1:spectral_points+1; % this data is unused in simulation

% Make spectral data files
spectral_slice=[freq' intensity'];
for counter=1:1:N_additions+1
    file=fullfile(path_to_save,path_simulation_data,sprintf('%d.txt',counter));
    save(file,'spectral_slice','-ascii')
end

% Make other files needed by LineShapeKin 3 simulation mode
point_list=titration_points';
save(fullfile(path_to_save,path_simulation_data,'points_list.txt'), 'point_list','-ascii');

a=1;
save(fullfile(path_to_save,path_simulation_data,'scale_factor.txt'), 'a','-ascii');

Rs=[R2_P;R2_PL];
save(fullfile(path_to_save,path_simulation_data,'manual_R2.txt'), 'Rs','-ascii');

ws=[wP; wPL];
save(fullfile(path_to_save,path_simulation_data,'manual_w0.txt'), 'ws','-ascii');












